
// base class using DMA
#include <cstring>

class baseDMA
{
    friend std::ostream & operator<<(std::ostream & os,
                                     const baseDMA & rs);
private:
    char * label;
    int rating;
public:
    baseDMA(const char * lbl = "nada", int r = 0);
    baseDMA(const baseDMA & rs);
    virtual ~baseDMA();
    baseDMA & operator=(const baseDMA & rs);

};

// derived class without DMA
class lacksDMA : public baseDMA {
//private:
public:
    enum { COL_LEN = 40 };
    char color[COL_LEN];
public:
    lacksDMA(const char * c = "blank", const char * lbl = "null",
             int r = 0);
    lacksDMA(const char * c, const baseDMA & rs);
    friend std::ostream & operator<<(std::ostream * os,
                                     const lacksDMA & rs);
};

/* Do you now have to define an explicit
destructor, copy constructor and assingment
operator for the lackDMA class?  NO!

The default destructor is fine.

Everything is inherited and fine.
*/

class hasDMA :public baseDMA {
friend std::ostream & operator<<(std::ostream * os,
                                     const hasDMA& rs);
//private:
public:
    char * style;  //use new in constructors
public:
    hasDMA(const char * s = "none", const char * lbl = "null",
             int r = 0);
    hasDMA(const char * s, const hasDMA & rs);
    hasDMA(const hasDMA & rs);
    ~hasDMA();
    hasDMA & operator=(const hasDMA & rs);

};

/* In this case you have to define an explicit destructor
copy constructor and assignment operator.
*/


baseDMA::~baseDMA() {
    delete [] label;
}


baseDMA::baseDMA(const char * lbl, int r)
{
    label = new char[std::strlen(lbl)+1];
    std::strcpy(label, lbl);
    rating = r;
}

baseDMA::baseDMA(const baseDMA & rs) {
    label = new char[std::strlen(rs.label)+1];
    std::strcpy(label,rs.label);
    rating = rs.rating;
}



/* Note that the member initializer listpasses a baseDMA reference to a
baseDMA constructor.  There is no baseDMA sontractor with a type hasDMA reference
parameter, but none is needed.  That's beacuse the baseDMA copyc onstructor has a
baseDMA reference param,eter, and a base class reference can refer to a derived type.
*/

baseDMA & baseDMA::operator=(const baseDMA & rs) {
    if(this == &rs)  return *this;
    delete [] label;
    label = new char[std::strlen(rs.label)+1];
    std::strcpy(label,rs.label);
    rating = rs.rating;
    return *this;
}

std::ostream & operator<<(std::ostream & os, const baseDMA & rs) {
    os << "Label: " << rs.label << std::endl;
    return os;
}

std::ostream & operator<<(std::ostream& os, const lacksDMA& ls) {
    os << (const baseDMA &) ls;
    os << "Color: " << ls.color << std::endl;
    return os;
}

std::ostream & operator<<(std::ostream & os, const hasDMA& hs) {
    os << (const baseDMA &) hs;
    os << "Style: " << hs.style << std::endl;
    return os;
}

lacksDMA::lacksDMA(const char * c, const char * lbl,
             int r) : baseDMA(lbl,r) {
    std::strncpy(color, c, 39);
    color[39] = '\0';
}

lacksDMA::lacksDMA(const char * c, const baseDMA & rs)
    : baseDMA(rs) {
    std:strncpy(color, c, COL_LEN-1);
    color[COL_LEN-1] = '\0';
};

//==========================
// hasDMA definitions

hasDMA::hasDMA(const char * s, const char * lbl, int r)
    : baseDMA(lbl, r)
{
    style = new char[std::strlen(s)+1];
    std:strcpy(style, s);
}
hasDMA::hasDMA(const char * s, const hasDMA & hs)
    : baseDMA(hs)
{
    style = new char[std::strlen(hs.style)+1];
    std:strcpy(style, hs.style);
}

hasDMA::hasDMA(const hasDMA & rs)
    : baseDMA(rs)
{
    style = new char[std::strlen(rs.style)+1];
    std::strcpy(style,rs.style);
}

hasDMA::~hasDMA() {
    delete [] style;
}

hasDMA & hasDMA::operator=(const hasDMA & hs) {
    if(this == &hs) return *this;
    baseDMA::operator=(hs);  //copy base portion
    delete [] style;
    style = new char[std::strlen(hs.style)+1];
    std:strcpy(style,hs.style);
    return *this;
}

//hasDMA & hasDMA::operator=(const hasDMA & hs) {
//    if(this == &rs)  return *this;
//    baseDMA::operator=(hs);  //copy base portion
//    /* That may look odd.  Use function notation
//    instead of operator notation so you can use
//    scope resolution.. In effect that means the folloing
//    *this = hs; // hasDMA::operator=() would be recursive*/
//    delete [] style;
//    style = new char[std::strlen(hs.style)+1];
//    std::strcopy(style,hs.style);
//    return *this;
//}
